﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("Single-objective Problem Definition", "Base class for custom single-objective problem definitions.")]
  [StorableClass]
  public abstract class SingleObjectiveProblemDefinition : ProblemDefinition, ISingleObjectiveProblemDefinition {
    [Storable(Name = "Storable")]
    private bool isMaximizationProblem;
    public bool IsMaximizationProblem {
      get { return isMaximizationProblem; }
      set {
        if (isMaximizationProblem == value) return;
        isMaximizationProblem = value;
        OnProblemDefinitionChanged();
      }
    }

    public abstract double Evaluate(IRandom random, Individual vector);

    public abstract void Analyze(Individual[] vectors, double[] qualities, ResultCollection results);

    [StorableConstructor]
    protected SingleObjectiveProblemDefinition(bool deserializing) : base(deserializing) { }
    protected SingleObjectiveProblemDefinition(SingleObjectiveProblemDefinition original, Cloner cloner)
      : base(original, cloner) {
      this.isMaximizationProblem = original.isMaximizationProblem;
    }
    protected SingleObjectiveProblemDefinition(Encoding encoding) : this(encoding, "SingleObjectiveProblemDefinition", null) { }
    protected SingleObjectiveProblemDefinition(Encoding encoding, string name) : this(encoding, name, null) { }
    protected SingleObjectiveProblemDefinition(Encoding encoding, string name, string description)
      : base(encoding) { }
  }
}
